import numpy
from os.path import dirname, isdir, isfile
import pandas
import torch
from torch.utils.data import DataLoader, RandomSampler, TensorDataset

class LabeledData:
    def __init__(self,
                 corpus,
                 label_column,
                 max_length=512, 
                 batch_size=16):
        self.corpus = corpus
        self.max_length = max_length
        self.batch_size = batch_size
        self._init_labels(label_column)

    def _init_labels(self, label_column):
        self.corpus.load()
        self.labels = self.corpus.data[label_column]
        self.unique = self.labels.unique()

    def load(self, bert):
        encoded_data = TensorDataset(*map(torch.tensor, self.train_for(bert)))
        return DataLoader(encoded_data,
                          sampler=RandomSampler(encoded_data),
                          batch_size=self.batch_size)

    def train_for(self, bert):
        texts = self.corpus.get_all('content')
        tokenized = bert.tokenizer(list(texts),
                                   padding='max_length',
                                   truncation='only_first')
        return (tokenized.input_ids,
                tokenized.attention_mask,
                bert.encoder.transform(self.labels))
